/* appdata.c generated by valac 0.56.19, the Vala compiler
 * generated from appdata.vala, do not modify */

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#define PARLA_APP_DATA_ID "io.github.trufae.Parla"
#define PARLA_APP_DATA_NAME "Parla"
#define PARLA_APP_DATA_DEVELOPER "pancake"
#define PARLA_APP_DATA_WEBSITE "https://github.com/trufae/parla"
#define PARLA_APP_DATA_ISSUE_URL "https://github.com/trufae/parla/issues"
#define PARLA_APP_DATA_COMMENTS "A Delta Chat client for GNOME"
#define PARLA_APP_DATA_RELEASE_NOTES "<ul>\n" \
"          <li>Bundle adwaita icons as fallback when not available</li>" \
"\n" \
"          <li>Fix memory leaks in media playback</li>\n" \
"          <li>Navigating thru chat pictures not limited to chat cache<" \
"/li>\n" \
"          <li>Fix memory leaks in chat switching and font/media scalin" \
"g</li>\n" \
"          <li>Rpc bridge is now fully async, solving random gtk freeze" \
"s</li>\n" \
"          <li>Select media playback speed</li>\n" \
"          <li>Add links in gallery panel</li>\n" \
"          <li>Link previews using the html og tags</li>\n" \
"          <li>Fix font contrast color for links</li>\n" \
"          <li>Filter tracking ids when pasting links (uBlock support)<" \
"/li>\n" \
"        </ul>\n" \
"      <p>Version 0.8.0</p><ul>\n" \
"          <li>Custom smtp/imap for the classic email accounts</li>\n" \
"          <li>Initial WebXDC support for Windows</li>\n" \
"          <li>Improve mentions in chats</li>\n" \
"          <li>Add an option to remove tracking IDs from links</li>\n" \
"          <li>Create new group cloning members from another</li>\n" \
"          <li>Transcribe voice messages at origin</li>\n" \
"          <li>Always keep the dock icon on macOS</li>\n" \
"          <li>Add support for archived chats</li>\n" \
"          <li>Fix slow window appearing on macOS</li>\n" \
"          <li>Share files by dropping them in the sidebar</li>\n" \
"        </ul>\n" \
"      <p>Version 0.7.8</p><ul>\n" \
"          <li>Better webxdc, app manager and capabilities</li>\n" \
"          <li>Redesigned settings dialog</li>\n" \
"          <li>Shared pinned messages (requires core v2.58)</li>\n" \
"        </ul>\n" \
"      <p>Version 0.7.6</p><ul>\n" \
"          <li>Experimental webxdc support</li>\n" \
"          <li>Minimize to Tray support for macOS</li>\n" \
"          <li>View larger avatars when clicked</li>\n" \
"          <li>Start in background for boot items</li>\n" \
"        </ul>\n" \
"      <p>Version 0.7.4</p><ul>\n" \
"          <li>Introduce the media gallery</li>\n" \
"          <li>Fix emoji bubbles on hidpi screens</li>\n" \
"          <li>Various composebar minor fixes</li>\n" \
"          <li>Support escaped markdown codeblocks</li>\n" \
"        </ul>\n" \
"      <p>Version 0.7.2</p><ul>\n" \
"          <li>Fix UI freeze</li>\n" \
"          <li>Toggle sticker animations to save cpu</li>\n" \
"          <li>Tap stickers to pause animations</li>\n" \
"          <li>Transcribe voice messages with WhisperCli</li>\n" \
"          <li>Long messages cannot be edited</li>\n" \
"          <li>Add code syntax highlighting</li>\n" \
"          <li>Preview md/txt/html files</li>\n" \
"          <li>Warn before posting a long uneditable message</li>\n" \
"        </ul>\n" \
"      <p>Version 0.7.0</p><ul>\n" \
"          <li>Improved large message visualization</li>\n" \
"          <li>Better stickers support</li>\n" \
"          <li>Support voice messages recording</li>\n" \
"          <li>Mediabar for consecutive voice messages</li>\n" \
"          <li>Improved scrolling experience</li>\n" \
"          <li>Better sidebar keyboard navigation</li>\n" \
"          <li>Configuratble AccountsPath</li>\n" \
"          <li>Fix rescheduled notifications bug</li>\n" \
"        </ul>\n" \
"      <p>Version 0.6.8</p><ul>\n" \
"          <li>Manage and send stickers</li>\n" \
"          <li>Fix composebar text alignment</li>\n" \
"          <li>Display QR codes in macOS</li>\n" \
"          <li>Show/hide toggle menu in tray</li>\n" \
"        </ul>\n" \
"      <p>Version 0.6.6</p><ul>\n" \
"          <li>Add new Workspace chat style</li>\n" \
"          <li>Better notification without dbus</li>\n" \
"          <li>Focus entry with Meta+L</li>\n" \
"          <li>Add combobox to select markdown mode</li>\n" \
"          <li>Support Windows</li>\n" \
"          <li>Lots of small UI/UX fixes</li>\n" \
"        </ul>\n" \
"      <p>Version 0.6.4</p><ul>\n" \
"          <li>Read receipt option moved into user profile</li>\n" \
"          <li>Strip markdown in message previews</li>\n" \
"          <li>Interactive markdown checkboxes</li>\n" \
"          <li>Delete context button in red</li>\n" \
"          <li>Reorganized settings dialog</li>\n" \
"          <li>Fix tray icon for XFCE and KDE</li>\n" \
"	  <li>Support attaching multiple files</li>\n" \
"	  <li>Display who reacted to a message</li>\n" \
"	  <li>Initial support for mentions</li>\n" \
"        </ul>\n" \
"      <p>Version 0.6.2</p><ul>\n" \
"          <li>Support draft messages</li>\n" \
"          <li>Hide notification message (privacy)</li>\n" \
"          <li>Initial mark-as-unread feature</li>\n" \
"          <li>Visualize online status with a green aura</li>\n" \
"          <li>Show QR code to invite chat</li>\n" \
"          <li>Show pin indicator in pinned messages</li>\n" \
"	  <li>Show date and time when scrolling in chats</li>\n" \
"        </ul>\n" \
"      <p>Version 0.6.0</p><ul>\n" \
"          <li>Delete/Forward multiple messages</li>\n" \
"          <li>Fix media bubble size in hidpi</li>\n" \
"          <li>Fix close window with disabled tray icon</li>\n" \
"          <li>Triple click selects all the text</li>\n" \
"          <li>Cleaner action menu</li>\n" \
"          <li>Show pin indicator in pinned messages</li>\n" \
"          <li>Fix crashes using UI from non-main thread</li>\n" \
"          <li>Allow to override system light/dark theme</li>\n" \
"        </ul>\n" \
"      <p>Version 0.5.8</p><ul>\n" \
"          <li>System program media playback as fallback</li>\n" \
"          <li>Fix the conversation scroll to top bug</li>\n" \
"        </ul>\n" \
"      <p>Version 0.5.6</p><ul>\n" \
"          <li>Add appImage</li>\n" \
"          <li>Switch between chats with Control+PageUp/Down</li>\n" \
"          <li>Highlight input field when in focus</li>\n" \
"          <li>Fix message bubble containing pictures</li>\n" \
"          <li>Fix audio messages playing muted</li>\n" \
"          <li>Workaround a macos/gtk4 clipboard bug</li>\n" \
"          <li>Add compile time option to hardcode jsonrpc server path<" \
"/li>\n" \
"        </ul>\n" \
"      <p>Version 0.5.4</p><ul>\n" \
"          <li>Discover relays to create new profile</li>\n" \
"          <li>Allow to rename group chats</li>\n" \
"          <li>Improve user profile usability</li>\n" \
"          <li>Fix coloring and tint for readability</li>\n" \
"          <li>Deprecate Deltachat desktop server finder</li>\n" \
"          <li>Don't clear input when attaching files</li>\n" \
"        </ul>\n" \
"      <p>Version 0.5.2</p><ul>\n" \
"          <li>Fix notification counters</li>\n" \
"          <li>View large messages</li>\n" \
"          <li>Fix ui refresh flood when syncing accounts</li>\n" \
"          <li>Show preview thumbnails for videos</li>\n" \
"          <li>Custom fonts and resize with keyboard/mouse</li>\n" \
"          <li>New keys to switch between accounts and chats</li>\n" \
"          <li>Render markdown tables</li>\n" \
"        </ul>\n" \
"      <p>Version 0.5.0</p><ul>\n" \
"          <li>Reduce UI refresh when syncing messages</li>\n" \
"          <li>Network proxy settings</li>\n" \
"          <li>Single emoji messages are displayed 5x larger</li>\n" \
"          <li>Edit contact names, and permit block/unblock</li>\n" \
"          <li>Fix some glitches in the multiline composer</li>\n" \
"          <li>Visual indicator for edited messages</li>\n" \
"          <li>Open chat when clicking in notifications</li>\n" \
"          <li>Improvements in the jsonrpc download/update</li>\n" \
"          <li>Fix shrink window for portrait mobile UX</li>\n" \
"          <li>Up arrow edits last message</li>\n" \
"          <li>Double colon raise the emoji picker</li>\n" \
"          <li>Incoming chat requests get accept/block buttons</li>\n" \
"          <li>Fix chat notification dismissal and display</li>\n" \
"        </ul>\n" \
"      <p>Version 0.4.0</p><ul>\n" \
"          <li>Media playback</li>\n" \
"          <li>Better input focus</li>\n" \
"          <li>Double ESC to cancel attachment</li>\n" \
"          <li>Configurable tray icon</li>\n" \
"          <li>Invite link creation and opening inside the app</li>\n" \
"          <li>Faster and non-blocking history loading</li>\n" \
"          <li>Label forwarded messages accordingly</li>\n" \
"          <li>Tag forwarded messages</li>\n" \
"          <li>Show notifications from all accounts</li>\n" \
"          <li>Support custom background color and gradient</li>\n" \
"        </ul>\n" \
"      <p>Version 0.3.0</p><ul>\n" \
"          <li>New icon and welcome screen</li>\n" \
"          <li>Better jsonrpc download onboarding</li>\n" \
"          <li>Allow to interrupt account creation before timeout</li>\n" \
"        </ul>\n" \
"      <p>Version 0.2.8</p><ul>\n" \
"          <li>Custom accent color</li>\n" \
"          <li>Bubble chat/irc view</li>\n" \
"          <li>Improved message reactions</li>\n" \
"          <li>Select default profile</li>\n" \
"          <li>Create channels</li>\n" \
"          <li>Toggle sidebar with keyboard shortcut</li>\n" \
"        </ul>\n" \
"      <p>Version 0.2.6</p><ul>\n" \
"          <li>Show user cache and server quota stats</li>\n" \
"          <li>Download and update chatmail jsonrpc server</li>\n" \
"          <li>No longer depends on Delta Chat Desktop</li>\n" \
"        </ul>\n" \
"      <p>Version 0.2.4</p><ul>\n" \
"          <li>Account creation</li>\n" \
"          <li>Enter chat via invite link</li>\n" \
"          <li>Manage relay profiles</li>\n" \
"          <li>Discover new relays from contacts</li>\n" \
"          <li>Profile lists available in the avatar circle</li>\n" \
"          <li>Support import/export secondary device</li>\n" \
"          <li>Prompt before destructive actions</li>\n" \
"          <li>Next/prev picture with arrows</li>\n" \
"        </ul>\n" \
"      <p>Version 0.2.2</p><ul>\n" \
"          <li>Focusable conversation list</li>\n" \
"          <li>Paste images from clipboard</li>\n" \
"          <li>Add delivery indicators</li>\n" \
"          <li>Add forward message action</li>\n" \
"          <li>Improve delete experience</li>\n" \
"          <li>Composebar with emoji-picker</li>\n" \
"        </ul>\n" \
"      <p>Version 0.2.1</p><ul>\n" \
"          <li>Add factory reset option in settings</li>\n" \
"          <li>Custom path to the JSONRPC server</li>\n" \
"          <li>Use ESC to focus the composebar</li>\n" \
"          <li>Faster conversation loads</li>\n" \
"        </ul>\n" \
"      <p>Version 0.2.0</p><p>This is the very first release</p>\n" \
"      "
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

VALA_EXTERN gchar** parla_app_data_developers (gint* result_length1);
VALA_EXTERN gchar* parla_app_data_release_notes (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

gchar**
parla_app_data_developers (gint* result_length1)
{
	gchar* _tmp0_;
	gchar** _tmp1_;
	gchar** _tmp2_;
	gint _tmp2__length1;
	gchar** result;
	_tmp0_ = g_strdup (PARLA_APP_DATA_DEVELOPER);
	_tmp1_ = g_new0 (gchar*, 1 + 1);
	_tmp1_[0] = _tmp0_;
	_tmp2_ = _tmp1_;
	_tmp2__length1 = 1;
	if (result_length1) {
		*result_length1 = _tmp2__length1;
	}
	result = _tmp2_;
	return result;
}

gchar*
parla_app_data_release_notes (void)
{
	gchar* _tmp0_;
	gchar* result;
	_tmp0_ = g_strdup (PARLA_APP_DATA_RELEASE_NOTES);
	result = _tmp0_;
	return result;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

