/* webxdc_security.c generated by valac 0.56.19, the Vala compiler
 * generated from webxdc_security.vala, do not modify */

#include <glib.h>
#include <stdlib.h>
#include <string.h>

#define DC_WEBXDC_SECURITY_SAFE_ALLOW_INTERNET FALSE
#define DC_WEBXDC_SECURITY_SAFE_ALLOW_WASM FALSE
#define DC_WEBXDC_SECURITY_SAFE_ALLOW_WEBGL FALSE
#define DC_WEBXDC_SECURITY_SAFE_ALLOW_HARDWARE_ACCELERATION FALSE
#define DC_WEBXDC_SECURITY_NO_WASM_CSP "script-src 'self' 'unsafe-inline' data: blob: http: https:"
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

VALA_EXTERN gchar* dc_webxdc_security_wasm_csp (gboolean allow_wasm);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

gchar*
dc_webxdc_security_wasm_csp (gboolean allow_wasm)
{
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_;
	gchar* result;
	if (allow_wasm) {
		_tmp0_ = NULL;
	} else {
		_tmp0_ = DC_WEBXDC_SECURITY_NO_WASM_CSP;
	}
	_tmp1_ = g_strdup (_tmp0_);
	result = _tmp1_;
	return result;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

